==> heptapod-rails-0.30.0rc1.tar.gz: heptapod-rails-0.30.0rc1/doc/user/mercurial.md <==
# Mercurial projects in Heptapod

The primary objective of Heptapod is to add Mercurial support to GitLab.

## Version Control System (VCS) types

In Heptapod, each Project has a VCS Type to describe the Version
Control System in use.

The VCS Type is selected upon creation, and a shortened form
is always displayed next on the Project home page, in the summary
area, next to the visibility (public/private/internal) icon.
It is also part of API responses, such as `GET /projects/:id`,
as the `vcs_type` field.

Instance Administrators
can select which VCS Types are allowed in the application settings
(Admin Area > Settings > General > Visibility and access controls).

We are currently using two distinct VCS types for Mercurial projects, to support our
effort towards "native" handling of Mercurial.

### "Mercurial" VCS type

- Value of the `vcs_type` field: `hg`.
- Label for Project summary: hg

This has been the default VCS type for new Projects since Heptapod 0.25 and was displayed
as "Mercurial (native)" before Heptapod 0.28.

Projects created right now under this mode should not need heavy and risky migrations
in the future. They are often referred to as "native Mercurial projects" in text resources,
such as documentation and issues.

The native Mercurial projects are exposed to the various GitLab components via the HGitaly
service.

As of Heptapod 0.29, content is still converted to Git by default, but this can be
disabled by feature flags. See the [administration documentation] for
more details on this. Future versions of Heptapod will stop
converting to Git, and the only difference visbile to users will be a speedup.

The important feature of native Mercurial projects is that the commit
hashes handled by GitLab are really the Mercurial changeset hashes.

### "Mercurial (legacy with Git conversion)" VCS type

- Value of the `vcs_type` field: `hg_git`.
- Label for Project summary: hg(legacy)

This VCS type was disabled by default for Project creation in Heptapod 0.25, but can
be enabled again in the application setttings.

This is the historical way to handle Mercurial repositories, exposing to the rest of
GitLab by maintaining a conversion to Git. Even though the server side Mercurial
repository is the single source of truth, this has many drawbacks.

With this type, most internal GitLab operations actually happen on Git
commits, resulting in many cases in wrong hashes being presented in
the user interface or API responses. These are actually hashes of the
corresponding Git commits, which cannot be guessed before hand by end
users. See [heptapod#6] for concrete cases of this.

## Migrating a Project to the (native) "Mercurial" VCS type

Warning: this is still considered experimental and *cannot be
 rollbacked*. You are advised to
migrate non-essential Projects only, and preferably test the migration
first. One way to do that is to export the relevant Project and
reimport it under a different name.

As of Heptapod 0.29, it is possible for end users to migrate their
existing projects from the legacy with Git conversion VCS type to the
native one, by issueing an API call.

Subsequent versions of Heptapod will also include a trigger in the Web
user interface (Project Settings > General > Advanced).

### API call to schedule the migration

This requires at least the Maintainer role


```plaintext
POST /projects/:id/hg_migrate_native
```

### Example


```shell
curl --request POST \
     --header "PRIVATE-TOKEN: <your_access_token>" \
     "https://heptapod.example/api/v4/projects/<your-project-ID>/hg_migrate_native"
```

### Details of the process

If accepted, the API call schedules the migration as a background job and
a HTTP 202 (accepted) response is immediately returned. The migration
itself should start in a matter of seconds, but this can be longer if
the server is busy.

The process then goes as follows:

- the Project is made read-only (archived) to prevent modification
  while the migration is running.
- A banner is displayed on the Project home page to explain that
  migration is in progress.
- The migration goes through all Merge Requests, Issues, Pipelines, etc.
  and replaces the internal references to Git Commit hashes by Mercurial
  changeset hashes.
- The Project is finally unarchived.

[administration documentation]: ../administration/mercurial.md
[heptapod#6]: https://foss.heptapod.net/heptapod/heptapod/-/issues/6

==> heptapod-rails-0.36.2.tar.gz: heptapod-rails-0.36.2/doc/user/mercurial.md <==
# Mercurial projects in Heptapod

The primary objective of Heptapod is to add Mercurial support to GitLab.

## Version Control System (VCS) types

In Heptapod, each Project has a VCS Type to describe the Version
Control System in use.

The VCS Type is selected upon creation, and a shortened form
is always displayed next on the Project home page, in the summary
area, next to the visibility (public/private/internal) icon.
It is also part of API responses, such as `GET /projects/:id`,
as the `vcs_type` field.

Instance Administrators
can select which VCS Types are allowed in the application settings
(Admin Area > Settings > General > Visibility and access controls).

We are currently using two distinct VCS types for Mercurial projects, to support our
effort towards "native" handling of Mercurial.

### "Mercurial" VCS type

- Value of the `vcs_type` field: `hg`.
- Label for Project summary: hg

This has been the default VCS type for new Projects since Heptapod 0.25 and was displayed
as "Mercurial (native)" before Heptapod 0.28.

Projects created right now under this mode should not need heavy and risky migrations
in the future. They are often referred to as "native Mercurial projects" in text resources,
such as documentation and issues. The important feature of native Mercurial projects is that the commit
hashes handled by GitLab are really the Mercurial changeset hashes.

The native Mercurial projects are exposed to the various GitLab components via the HGitaly
service.

#### Internal conversion to Git for native Mercurial projects

The point of native Mercurial projects is eventually not to depend on any internal conversion to Git to
provide the whole set of GitLab features. However, there are still cases when this conversion happens.

Some users found the legacy Git conversion to be useful to
[mirror repositories to Git-only services](https://foss.heptapod.net/heptapod/heptapod/-/issues/125). We plan to eventually provide a
[better Git push-mirroring capability](https://foss.heptapod.net/heptapod/heptapod/-/issues/355),
but it will probably involve different branch names than the legacy conversion, and it is possible that there
are other use-cases that we are not aware of.
Therefore, from version 0.36.2 onwards, to ease the transition and prevent breaking useful functionality, Heptapod comes with an exception setting: if "Legacy conversion to Git" is selected in Settings > Repository > Mercurial Settings, the legacy conversion still happens, even in later versions that would otherwise not convert native projects to Git any more.

As of Heptapod 0.36, content is still converted to Git by default, but this can be
disabled by feature flags. See the [administration documentation] for
more details on this. Heptapod will stop converting to Git by default in version 0.38, and this can still
be reverted by feature flags.

### "Mercurial (legacy with Git conversion)" VCS type

- Value of the `vcs_type` field: `hg_git`.
- Label for Project summary: hg(legacy)

This VCS type was disabled by default for Project creation in Heptapod 0.25, but can
be enabled again in the application setttings.

This is the historical way to handle Mercurial repositories, exposing to the rest of
GitLab by maintaining a conversion to Git. Even though the server side Mercurial
repository is the single source of truth, this has many drawbacks.

With this type, most internal GitLab operations actually happen on Git
commits, resulting in many cases in wrong hashes being presented in
the user interface or API responses. These are actually hashes of the
corresponding Git commits, which cannot be guessed before hand by end
users. See [heptapod#6] for concrete cases of this.

## Migrating a Project to the (native) "Mercurial" VCS type

Warning: this is still considered experimental and *cannot be
 rollbacked*. You are advised to
migrate non-essential Projects only, and preferably test the migration
first. One way to do that is to export the relevant Project and
reimport it under a different name.

As of Heptapod 0.29, it is possible for end users to migrate their
existing projects from the legacy with Git conversion VCS type to the
native one, by issueing an API call.

Subsequent versions of Heptapod will also include a trigger in the Web
user interface (Project Settings > General > Advanced).

### API call to schedule the migration

This requires at least the Maintainer role


```plaintext
POST /projects/:id/hg_migrate_native
```

### Example


```shell
curl --request POST \
     --header "PRIVATE-TOKEN: <your_access_token>" \
     "https://heptapod.example/api/v4/projects/<your-project-ID>/hg_migrate_native"
```

### Details of the process

If accepted, the API call schedules the migration as a background job and
a HTTP 202 (accepted) response is immediately returned. The migration
itself should start in a matter of seconds, but this can be longer if
the server is busy.

The process then goes as follows:

- the Project is made read-only (archived) to prevent modification
  while the migration is running.
- A banner is displayed on the Project home page to explain that
  migration is in progress.
- The migration goes through all Merge Requests, Issues, Pipelines, etc.
  and replaces the internal references to Git Commit hashes by Mercurial
  changeset hashes.
- The Project is finally unarchived.

[administration documentation]: ../administration/mercurial.md
[heptapod#6]: https://foss.heptapod.net/heptapod/heptapod/-/issues/6
